@@ -52,6 +52,7 @@ struct _GeeArrayListIteratorClass {
 struct _GeeArrayListPrivate {
 	gpointer* _items;
 	gint _items_length1;
+	gint _items_size;
 	gint _size;
 	GEqualFunc _equal_func;
 	gint _stamp;
@@ -80,6 +81,7 @@ static void gee_array_list_real_insert (GeeList* base, gint index, gconstpointer
 static gboolean gee_array_list_real_remove (GeeCollection* base, gconstpointer item);
 static void gee_array_list_real_remove_at (GeeList* base, gint index);
 static void gee_array_list_real_clear (GeeCollection* base);
+static GeeList* gee_array_list_real_slice (GeeList* base, gint start, gint stop);
 static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta);
 static void gee_array_list_grow_if_needed (GeeArrayList* self, gint new_count);
 static void gee_array_list_set_capacity (GeeArrayList* self, gint value);
@@ -154,28 +156,28 @@ GeeArrayList* gee_array_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDest
 
 static GType gee_array_list_real_get_element_type (GeeIterable* base) {
 	GeeArrayList * self;
-	self = GEE_ARRAY_LIST (base);
+	self = (GeeArrayList*) base;
 	return self->priv->g_type;
 }
 
 
 static GeeIterator* gee_array_list_real_iterator (GeeIterable* base) {
 	GeeArrayList * self;
-	self = GEE_ARRAY_LIST (base);
-	return GEE_ITERATOR (gee_array_list_iterator_new (self->priv->g_type, ((GBoxedCopyFunc) (self->priv->g_dup_func)), self->priv->g_destroy_func, self));
+	self = (GeeArrayList*) base;
+	return (GeeIterator*) gee_array_list_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self);
 }
 
 
 static gboolean gee_array_list_real_contains (GeeCollection* base, gconstpointer item) {
 	GeeArrayList * self;
-	self = GEE_ARRAY_LIST (base);
-	return (gee_list_index_of (GEE_LIST (self), item) != -1);
+	self = (GeeArrayList*) base;
+	return (gee_list_index_of ((GeeList*) self, item) != (-1));
 }
 
 
 static gint gee_array_list_real_index_of (GeeList* base, gconstpointer item) {
 	GeeArrayList * self;
-	self = GEE_ARRAY_LIST (base);
+	self = (GeeArrayList*) base;
 	{
 		gint index;
 		index = 0;
@@ -191,23 +193,37 @@ static gint gee_array_list_real_index_of (GeeList* base, gconstpointer item) {
 
 static gpointer gee_array_list_real_get (GeeList* base, gint index) {
 	GeeArrayList * self;
-	gconstpointer _tmp0;
-	self = GEE_ARRAY_LIST (base);
-	g_assert (index >= 0 && index < self->priv->_size);
-	_tmp0 = NULL;
-	return (_tmp0 = self->priv->_items[index], (_tmp0 == NULL || self->priv->g_dup_func == NULL ? ((gpointer) (_tmp0)) : self->priv->g_dup_func (((gpointer) (_tmp0)))));
+	gboolean _tmp0;
+	gconstpointer _tmp1;
+	self = (GeeArrayList*) base;
+	_tmp0 = FALSE;
+	if (index >= 0) {
+		_tmp0 = index < self->priv->_size;
+	} else {
+		_tmp0 = FALSE;
+	}
+	g_assert (_tmp0);
+	_tmp1 = NULL;
+	return (_tmp1 = self->priv->_items[index], ((_tmp1 == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp1) : self->priv->g_dup_func ((gpointer) _tmp1));
 }
 
 
 static void gee_array_list_real_set (GeeList* base, gint index, gconstpointer item) {
 	GeeArrayList * self;
-	gpointer _tmp1;
-	gconstpointer _tmp0;
-	self = GEE_ARRAY_LIST (base);
-	g_assert (index >= 0 && index < self->priv->_size);
+	gboolean _tmp0;
+	gpointer _tmp2;
+	gconstpointer _tmp1;
+	self = (GeeArrayList*) base;
+	_tmp0 = FALSE;
+	if (index >= 0) {
+		_tmp0 = index < self->priv->_size;
+	} else {
+		_tmp0 = FALSE;
+	}
+	g_assert (_tmp0);
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	_tmp0 = NULL;
-	self->priv->_items[index] = (_tmp1 = (_tmp0 = item, (_tmp0 == NULL || self->priv->g_dup_func == NULL ? ((gpointer) (_tmp0)) : self->priv->g_dup_func (((gpointer) (_tmp0))))), (self->priv->_items[index] == NULL || self->priv->g_destroy_func == NULL ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL))), _tmp1);
+	self->priv->_items[index] = (_tmp2 = (_tmp1 = item, ((_tmp1 == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp1) : self->priv->g_dup_func ((gpointer) _tmp1)), ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp2);
 }
 
 
@@ -216,14 +232,14 @@ static gboolean gee_array_list_real_add (GeeCollection* base, gconstpointer item
 	gint _tmp2;
 	gpointer _tmp1;
 	gconstpointer _tmp0;
-	self = GEE_ARRAY_LIST (base);
+	self = (GeeArrayList*) base;
 	if (self->priv->_size == self->priv->_items_length1) {
 		gee_array_list_grow_if_needed (self, 1);
 	}
 	_tmp1 = NULL;
 	_tmp0 = NULL;
 	_tmp2 = self->priv->_size++;
-	self->priv->_items[_tmp2] = (_tmp1 = (_tmp0 = item, (_tmp0 == NULL || self->priv->g_dup_func == NULL ? ((gpointer) (_tmp0)) : self->priv->g_dup_func (((gpointer) (_tmp0))))), (self->priv->_items[_tmp2] == NULL || self->priv->g_destroy_func == NULL ? NULL : (self->priv->_items[_tmp2] = (self->priv->g_destroy_func (self->priv->_items[_tmp2]), NULL))), _tmp1);
+	self->priv->_items[_tmp2] = (_tmp1 = (_tmp0 = item, ((_tmp0 == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp0) : self->priv->g_dup_func ((gpointer) _tmp0)), ((self->priv->_items[_tmp2] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[_tmp2] = (self->priv->g_destroy_func (self->priv->_items[_tmp2]), NULL)), _tmp1);
 	self->priv->_stamp++;
 	return TRUE;
 }
@@ -231,30 +247,37 @@ static gboolean gee_array_list_real_add (GeeCollection* base, gconstpointer item
 
 static void gee_array_list_real_insert (GeeList* base, gint index, gconstpointer item) {
 	GeeArrayList * self;
-	gpointer _tmp1;
-	gconstpointer _tmp0;
-	self = GEE_ARRAY_LIST (base);
-	g_assert (index >= 0 && index <= self->priv->_size);
+	gboolean _tmp0;
+	gpointer _tmp2;
+	gconstpointer _tmp1;
+	self = (GeeArrayList*) base;
+	_tmp0 = FALSE;
+	if (index >= 0) {
+		_tmp0 = index <= self->priv->_size;
+	} else {
+		_tmp0 = FALSE;
+	}
+	g_assert (_tmp0);
 	if (self->priv->_size == self->priv->_items_length1) {
 		gee_array_list_grow_if_needed (self, 1);
 	}
 	gee_array_list_shift (self, index, 1);
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	_tmp0 = NULL;
-	self->priv->_items[index] = (_tmp1 = (_tmp0 = item, (_tmp0 == NULL || self->priv->g_dup_func == NULL ? ((gpointer) (_tmp0)) : self->priv->g_dup_func (((gpointer) (_tmp0))))), (self->priv->_items[index] == NULL || self->priv->g_destroy_func == NULL ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL))), _tmp1);
+	self->priv->_items[index] = (_tmp2 = (_tmp1 = item, ((_tmp1 == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp1) : self->priv->g_dup_func ((gpointer) _tmp1)), ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp2);
 	self->priv->_stamp++;
 }
 
 
 static gboolean gee_array_list_real_remove (GeeCollection* base, gconstpointer item) {
 	GeeArrayList * self;
-	self = GEE_ARRAY_LIST (base);
+	self = (GeeArrayList*) base;
 	{
 		gint index;
 		index = 0;
 		for (; index < self->priv->_size; index++) {
 			if (self->priv->_equal_func (self->priv->_items[index], item)) {
-				gee_list_remove_at (GEE_LIST (self), index);
+				gee_list_remove_at ((GeeList*) self, index);
 				return TRUE;
 			}
 		}
@@ -265,11 +288,18 @@ static gboolean gee_array_list_real_remove (GeeCollection* base, gconstpointer i
 
 static void gee_array_list_real_remove_at (GeeList* base, gint index) {
 	GeeArrayList * self;
-	gpointer _tmp0;
-	self = GEE_ARRAY_LIST (base);
-	g_assert (index >= 0 && index < self->priv->_size);
-	_tmp0 = NULL;
-	self->priv->_items[index] = (_tmp0 = NULL, (self->priv->_items[index] == NULL || self->priv->g_destroy_func == NULL ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL))), _tmp0);
+	gboolean _tmp0;
+	gpointer _tmp1;
+	self = (GeeArrayList*) base;
+	_tmp0 = FALSE;
+	if (index >= 0) {
+		_tmp0 = index < self->priv->_size;
+	} else {
+		_tmp0 = FALSE;
+	}
+	g_assert (_tmp0);
+	_tmp1 = NULL;
+	self->priv->_items[index] = (_tmp1 = NULL, ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp1);
 	gee_array_list_shift (self, index + 1, -1);
 	self->priv->_stamp++;
 }
@@ -277,14 +307,14 @@ static void gee_array_list_real_remove_at (GeeList* base, gint index) {
 
 static void gee_array_list_real_clear (GeeCollection* base) {
 	GeeArrayList * self;
-	self = GEE_ARRAY_LIST (base);
+	self = (GeeArrayList*) base;
 	{
 		gint index;
 		index = 0;
 		for (; index < self->priv->_size; index++) {
 			gpointer _tmp0;
 			_tmp0 = NULL;
-			self->priv->_items[index] = (_tmp0 = NULL, (self->priv->_items[index] == NULL || self->priv->g_destroy_func == NULL ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL))), _tmp0);
+			self->priv->_items[index] = (_tmp0 = NULL, ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp0);
 		}
 	}
 	self->priv->_size = 0;
@@ -292,9 +322,45 @@ static void gee_array_list_real_clear (GeeCollection* base) {
 }
 
 
+static GeeList* gee_array_list_real_slice (GeeList* base, gint start, gint stop) {
+	GeeArrayList * self;
+	GeeArrayList* slice;
+	self = (GeeArrayList*) base;
+	g_return_val_if_fail (start <= stop, NULL);
+	g_return_val_if_fail (start >= 0, NULL);
+	g_return_val_if_fail (stop <= gee_collection_get_size ((GeeCollection*) self), NULL);
+	slice = gee_array_list_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self->priv->_equal_func);
+	{
+		gint i;
+		i = start;
+		for (; i < stop; i++) {
+			gpointer _tmp0;
+			_tmp0 = NULL;
+			gee_collection_add ((GeeCollection*) slice, _tmp0 = gee_list_get ((GeeList*) self, i));
+			((_tmp0 == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp0 = (self->priv->g_destroy_func (_tmp0), NULL));
+		}
+	}
+	return (GeeList*) slice;
+}
+
+
 static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta) {
-	g_return_if_fail (GEE_IS_ARRAY_LIST (self));
-	g_assert (start >= 0 && start <= self->priv->_size && start >= -delta);
+	gboolean _tmp0;
+	gboolean _tmp1;
+	g_return_if_fail (self != NULL);
+	_tmp0 = FALSE;
+	_tmp1 = FALSE;
+	if (start >= 0) {
+		_tmp1 = start <= self->priv->_size;
+	} else {
+		_tmp1 = FALSE;
+	}
+	if (_tmp1) {
+		_tmp0 = start >= (-delta);
+	} else {
+		_tmp0 = FALSE;
+	}
+	g_assert (_tmp0);
 	_vala_array_move (self->priv->_items, sizeof (gpointer), start, start + delta, self->priv->_size - start);
 	self->priv->_size = self->priv->_size + (delta);
 }
@@ -302,38 +368,45 @@ static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta) {
 
 static void gee_array_list_grow_if_needed (GeeArrayList* self, gint new_count) {
 	gint minimum_size;
-	g_return_if_fail (GEE_IS_ARRAY_LIST (self));
+	g_return_if_fail (self != NULL);
 	g_assert (new_count >= 0);
 	minimum_size = self->priv->_size + new_count;
 	if (minimum_size > self->priv->_items_length1) {
+		gint _tmp0;
+		_tmp0 = 0;
+		if (new_count > self->priv->_items_length1) {
+			_tmp0 = minimum_size;
+		} else {
+			_tmp0 = 2 * self->priv->_items_length1;
+		}
 		/* double the capacity unless we add even more items at this time*/
-		gee_array_list_set_capacity (self, (new_count > self->priv->_items_length1 ? minimum_size : 2 * self->priv->_items_length1));
+		gee_array_list_set_capacity (self, _tmp0);
 	}
 }
 
 
 static void gee_array_list_set_capacity (GeeArrayList* self, gint value) {
 	gint _tmp0;
-	g_return_if_fail (GEE_IS_ARRAY_LIST (self));
+	g_return_if_fail (self != NULL);
 	g_assert (value >= self->priv->_size);
 	_tmp0 = value;
 	self->priv->_items = g_renew (gpointer, self->priv->_items, value);
-	(_tmp0 > self->priv->_items_length1 ? memset (self->priv->_items + self->priv->_items_length1, 0, sizeof (gpointer) * (_tmp0 - self->priv->_items_length1)) : NULL);
+	(_tmp0 > self->priv->_items_length1) ? memset (self->priv->_items + self->priv->_items_length1, 0, sizeof (gpointer) * (_tmp0 - self->priv->_items_length1)) : NULL;
 	self->priv->_items_length1 = _tmp0;
 }
 
 
 static gint gee_array_list_real_get_size (GeeCollection* base) {
 	GeeArrayList* self;
-	self = GEE_ARRAY_LIST (base);
+	self = (GeeArrayList*) base;
 	return self->priv->_size;
 }
 
 
 void gee_array_list_set_equal_func (GeeArrayList* self, GEqualFunc value) {
-	g_return_if_fail (GEE_IS_ARRAY_LIST (self));
+	g_return_if_fail (self != NULL);
 	self->priv->_equal_func = value;
-	g_object_notify (((GObject *) (self)), "equal-func");
+	g_object_notify ((GObject *) self, "equal-func");
 }
 
 
@@ -341,7 +414,7 @@ static GeeArrayListIterator* gee_array_list_iterator_construct (GType object_typ
 	GParameter * __params;
 	GParameter * __params_it;
 	GeeArrayListIterator * self;
-	g_return_val_if_fail (GEE_IS_ARRAY_LIST (list), NULL);
+	g_return_val_if_fail (list != NULL, NULL);
 	__params = g_new0 (GParameter, 3);
 	__params_it = __params;
 	__params_it->name = "g-type";
@@ -374,7 +447,7 @@ static GeeArrayListIterator* gee_array_list_iterator_new (GType g_type, GBoxedCo
 
 static gboolean gee_array_list_iterator_real_next (GeeIterator* base) {
 	GeeArrayListIterator * self;
-	self = GEE_ARRAY_LIST_ITERATOR (base);
+	self = (GeeArrayListIterator*) base;
 	g_assert (self->_stamp == self->priv->_list->priv->_stamp);
 	if (self->priv->_index < self->priv->_list->priv->_size) {
 		self->priv->_index++;
@@ -385,29 +458,37 @@ static gboolean gee_array_list_iterator_real_next (GeeIterator* base) {
 
 static gpointer gee_array_list_iterator_real_get (GeeIterator* base) {
 	GeeArrayListIterator * self;
-	self = GEE_ARRAY_LIST_ITERATOR (base);
+	gboolean _tmp0;
+	self = (GeeArrayListIterator*) base;
 	g_assert (self->_stamp == self->priv->_list->priv->_stamp);
-	if (self->priv->_index < 0 || self->priv->_index >= self->priv->_list->priv->_size) {
+	_tmp0 = FALSE;
+	if (self->priv->_index < 0) {
+		_tmp0 = TRUE;
+	} else {
+		_tmp0 = self->priv->_index >= self->priv->_list->priv->_size;
+	}
+	if (_tmp0) {
 		return NULL;
 	}
-	return gee_list_get (GEE_LIST (self->priv->_list), self->priv->_index);
+	return gee_list_get ((GeeList*) self->priv->_list, self->priv->_index);
 }
 
 
 static void gee_array_list_iterator_set_list (GeeArrayListIterator* self, GeeArrayList* value) {
 	GeeArrayList* _tmp1;
 	GeeArrayList* _tmp0;
-	g_return_if_fail (GEE_ARRAY_LIST_IS_ITERATOR (self));
+	g_return_if_fail (self != NULL);
 	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->_list = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_list == NULL ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL))), _tmp1);
+	self->priv->_list = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_list == NULL) ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL)), _tmp1);
 	self->_stamp = self->priv->_list->priv->_stamp;
-	g_object_notify (((GObject *) (self)), "list");
+	g_object_notify ((GObject *) self, "list");
 }
 
 
 static void gee_array_list_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
 	GeeArrayListIterator * self;
+	gpointer boxed;
 	self = GEE_ARRAY_LIST_ITERATOR (object);
 	switch (property_id) {
 		default:
@@ -424,6 +505,9 @@ static void gee_array_list_iterator_set_property (GObject * object, guint proper
 		case GEE_ARRAY_LIST_ITERATOR_LIST:
 		gee_array_list_iterator_set_list (self, g_value_get_object (value));
 		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
 		case GEE_ARRAY_LIST_ITERATOR_G_TYPE:
 		self->priv->g_type = g_value_get_gtype (value);
 		break;
@@ -433,9 +517,6 @@ static void gee_array_list_iterator_set_property (GObject * object, guint proper
 		case GEE_ARRAY_LIST_ITERATOR_G_DESTROY_FUNC:
 		self->priv->g_destroy_func = g_value_get_pointer (value);
 		break;
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
 	}
 }
 
@@ -470,7 +551,7 @@ static void gee_array_list_iterator_instance_init (GeeArrayListIterator * self)
 static void gee_array_list_iterator_finalize (GObject* obj) {
 	GeeArrayListIterator * self;
 	self = GEE_ARRAY_LIST_ITERATOR (obj);
-	(self->priv->_list == NULL ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL)));
+	(self->priv->_list == NULL) ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL));
 	G_OBJECT_CLASS (gee_array_list_iterator_parent_class)->finalize (obj);
 }
 
@@ -489,10 +570,11 @@ static GType gee_array_list_iterator_get_type (void) {
 
 static void gee_array_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
 	GeeArrayList * self;
+	gpointer boxed;
 	self = GEE_ARRAY_LIST (object);
 	switch (property_id) {
 		case GEE_ARRAY_LIST_SIZE:
-		g_value_set_int (value, gee_collection_get_size (GEE_COLLECTION (self)));
+		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
 		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -508,6 +590,9 @@ static void gee_array_list_set_property (GObject * object, guint property_id, co
 		case GEE_ARRAY_LIST_EQUAL_FUNC:
 		gee_array_list_set_equal_func (self, g_value_get_pointer (value));
 		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
 		case GEE_ARRAY_LIST_G_TYPE:
 		self->priv->g_type = g_value_get_gtype (value);
 		break;
@@ -517,9 +602,6 @@ static void gee_array_list_set_property (GObject * object, guint property_id, co
 		case GEE_ARRAY_LIST_G_DESTROY_FUNC:
 		self->priv->g_destroy_func = g_value_get_pointer (value);
 		break;
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
 	}
 }
 
@@ -562,6 +644,7 @@ static void gee_array_list_gee_list_interface_init (GeeListIface * iface) {
 	iface->set = gee_array_list_real_set;
 	iface->insert = gee_array_list_real_insert;
 	iface->remove_at = gee_array_list_real_remove_at;
+	iface->slice = gee_array_list_real_slice;
 }
 
 
@@ -576,7 +659,7 @@ static void gee_array_list_instance_init (GeeArrayList * self) {
 static void gee_array_list_finalize (GObject* obj) {
 	GeeArrayList * self;
 	self = GEE_ARRAY_LIST (obj);
-	self->priv->_items = (_vala_array_free (self->priv->_items, self->priv->_items_length1, ((GDestroyNotify) (self->priv->g_destroy_func))), NULL);
+	self->priv->_items = (_vala_array_free (self->priv->_items, self->priv->_items_length1, (GDestroyNotify) self->priv->g_destroy_func), NULL);
 	G_OBJECT_CLASS (gee_array_list_parent_class)->finalize (obj);
 }
 
@@ -598,16 +681,12 @@ GType gee_array_list_get_type (void) {
 
 
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
-	if (array != NULL && destroy_func != NULL) {
+	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
-		if (array_length >= 0)
 		for (i = 0; i < array_length; i = i + 1) {
-			if (((gpointer*) (array))[i] != NULL)
-			destroy_func (((gpointer*) (array))[i]);
-		}
-		else
-		for (i = 0; ((gpointer*) (array))[i] != NULL; i = i + 1) {
-			destroy_func (((gpointer*) (array))[i]);
+			if (((gpointer*) array)[i] != NULL) {
+				destroy_func (((gpointer*) array)[i]);
+			}
 		}
 	}
 	g_free (array);
@@ -615,11 +694,11 @@ static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify
 
 
 static void _vala_array_move (gpointer array, gsize element_size, gint src, gint dest, gint length) {
-	g_memmove (((char*) (array)) + dest * element_size, ((char*) (array)) + src * element_size, length * element_size);
+	g_memmove (((char*) array) + (dest * element_size), ((char*) array) + (src * element_size), length * element_size);
 	if (src < dest) {
-		memset (((char*) (array)) + src * element_size, 0, (dest - src) * element_size);
+		memset (((char*) array) + (src * element_size), 0, (dest - src) * element_size);
 	} else {
-		memset (((char*) (array)) + (dest + length) * element_size, 0, (src - dest) * element_size);
+		memset (((char*) array) + ((dest + length) * element_size), 0, (src - dest) * element_size);
 	}
 }
 
